@@ -1,3 +1,5 @@
+use std::error::Error;
+use std::fmt;
 use std::path::Path;
 
 use crate::deploy::DeployTarget;
@@ -9,7 +11,7 @@ use crate::terminal::message::{Message, StdOut};
 use crate::upload;
 
 use anyhow::{anyhow, Result};
-use reqwest::Url;
+use reqwest::{StatusCode, Url};
 use serde::{Deserialize, Serialize};
 use serde_json::json;
 
@@ -49,8 +51,12 @@ pub(super) fn upload(
         .post(&address)
         .header("cf-preview-upload-config-token", session_token)
         .multipart(script_upload_form)
-        .send()?
-        .error_for_status()?;
+        .send()?;
+
+    if response.status() == StatusCode::BAD_REQUEST {
+        return Err(BadRequestError(crate::format_api_errors(response.text()?)).into());
+    }
+    let response = response.error_for_status()?;
 
     if !to_delete.is_empty() {
         if verbose {
@@ -71,6 +77,7 @@ pub(super) fn upload(
 pub struct Session {
     pub host: String,
     pub websocket_url: Url,
+    pub prewarm_url: Url,
     pub preview_token: String,
 }
 
@@ -100,16 +107,17 @@ impl Session {
         let response = client.get(exchange_url).send()?.error_for_status()?;
         let text = &response.text()?;
         let response: InspectorV4ApiResponse = serde_json::from_str(text)?;
-        let full_url = format!(
+        let websocket_url = format!(
             "{}?{}={}",
             &response.inspector_websocket, "cf_workers_preview_token", &response.token
-        );
-        let websocket_url = Url::parse(&full_url)?;
+        )
+        .parse()?;
         let preview_token = response.token;
 
         Ok(Session {
             host,
             websocket_url,
+            prewarm_url: response.prewarm.parse()?,
             preview_token,
         })
     }
@@ -177,6 +185,7 @@ struct SessionV4ApiResponse {
 #[derive(Debug, Serialize, Deserialize)]
 struct InspectorV4ApiResponse {
     pub inspector_websocket: String,
+    pub prewarm: String,
     pub token: String,
 }
 
@@ -189,3 +198,14 @@ struct Preview {
 struct PreviewV4ApiResponse {
     pub result: Preview,
 }
+
+#[derive(Debug)]
+pub(crate) struct BadRequestError(pub(crate) String);
+
+impl Error for BadRequestError {}
+
+impl fmt::Display for BadRequestError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        self.0.fmt(f)
+    }
+}
